function chart_pos_order_product (widget) {
    "use strict";

    var model = openerp;

    widget.ChartPosOrderProductWidget = widget.Base.extend({
        template: 'ChartPosOrderProduct',
        data: [],

        events: {
            
        },
        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },
        start: function () {
            var self = this;
            self.fetchInitial();
        },
        fetchInitial: function() {
            var self = this;
            self.$el.block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#FAFAFA'
                }
            });
            self.$el.find('.widget-content.widget-loading').css('display','flex');

            self.fecthPosOrder().then(function (PosOrder) {
                return PosOrder;
            }).then(function (PosOrder) {
                self.PosOrder = PosOrder;
                return self.fecthPosOrderLine(PosOrder);
            }).then(function (PosOrderLine) {
                self.PosOrderLine = PosOrderLine;
                return self.fecthProductProduct(PosOrderLine);
            }).then(function(productProduct){
                self.productProduct = productProduct;
                return self.fetchProductRanking();
            });
        },

        // Usuario Logeado
        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        // Lista de Graficos disponibles para el usuario
        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','chart_ids'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        // Obtener detalles de la lista de graficos
        fetchChartList: function(chart_ids) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name'];
            var domain = [['id','in',chart_ids]];
            var ChartList = new model.web.Model('chart.list');
            ChartList.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },
        
        // Obtener Pedido
        fecthPosOrder: function() {
            var self = this;
            var desde = moment().format('YYYY-MM-01 00:00:00');
            var hasta = moment().add(1,'months').format('YYYY-MM-01 00:00:00');
            var defer = $.Deferred();
            var fields = ['id', 'lines', 'date_order'];
            var domain = [['date_order', '>=', desde], ['date_order', '<', hasta], ['state', 'in', ['paid','done','invoiced']]];
            var PosOrder = new model.web.Model('pos.order');
            PosOrder.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },
        // Obtener linea de la factura
        fecthPosOrderLine: function(PosOrder) {
            var self = this;
            var defer = $.Deferred();
            var lines = _.flatten(_.map(PosOrder, function (item) {
                return item.lines;
            }));
            var fields = ['id', 'product_id', 'qty'];
            var domain = [['id','in', lines]];
            var PosOrderLine = new model.web.Model('pos.order.line');
            PosOrderLine.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },
        // Obtener Productos
        fecthProductProduct: function (PosOrderLine) {
            var self = this;
            var defer = $.Deferred();
            var product_id = _.flatten(_.map(PosOrderLine,function (item) {
                return item.product_id[0];
            }));
            var fields = ['id', 'name_template', 'type','attribute_str'];
            var domain = [['id', 'in', product_id]];
            var ProductProduct = new model.web.Model('product.product');
            ProductProduct.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },
        // get line
        getPosOrderLine: function (product_id) {
            var self = this;
            return _.flatten(_.filter(self.PosOrderLine,function (item) {
                return item.product_id[0] === product_id
            }));
        },
        // Generear el ranking
        fetchProductRanking: function() {
            var self = this;
            var itemProduct;
            var itemLine;
            var ranking = [];
            var cat = 0;
            var attribute;
            var lineUnik;
            for (var i = 0; i < self.productProduct.length; i++) {
                itemProduct = self.productProduct[i];
                itemLine = self.getPosOrderLine(itemProduct.id);
                if (itemLine.length > 0) {
                    cat = _.reduce(_.map(itemLine, function (map) {
                        return map.qty
                    }), function (meno,num) {
                        return meno + num
                    }, 0);

                    lineUnik = itemLine.shift();

                    if(itemProduct.attribute_str){
                        attribute = '*' + itemProduct.attribute_str;
                    }else{
                        attribute = ' ';
                    }
                    ranking.push({
                        product: itemProduct.name_template + ' ' + attribute,
                        qty: cat,
                        id: lineUnik.product_id[0],
                        type: itemProduct.type
                    });
                }
            }
            ranking.sort(function(a, b) {
                return b.qty - a.qty
            });
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(ranking);
        },     
        // Generar Grafico
        fetchChart: function (ranking){
            var self = this;
            var label = [];
            var bodyProducto = [];
            var labelService = [];
            var bodyService = [];
            var rank = 7;
            var item;

            for (var i = 0; i < rank; i++) {
                if (ranking[i]){
                    item = ranking[i];
                }else{
                    item = {};
                    item.product = "N/A";
                    item.qty = 0;
                }
                var name = item.product.split('*');
                if (name.length === 1) {
                        label.push(name[0]);
                }else{
                    label.push(name[0]+"("+name[name.length-1]+")");
                }
                if (item.type === 'product'){
                    bodyService.push(0);
                    bodyProducto.push(item.qty);
                }
                if (item.type === 'service'){
                    bodyService.push(item.qty);
                    bodyProducto.push(0);
                }
            }

            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'line',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Productos ',
                            borderWidth: 2,
                            backgroundColor: '#e3f2fd',
                            borderColor: '#64b5f6',
                            data: bodyProducto,
                            fill: false,
                        },
                        {
                            label: 'Servicios ',
                            borderWidth: 2,
                            backgroundColor: '#c8e6c9',
                            borderColor: '#66bb6a',
                            borderDash: [5, 5],
                            data: bodyService,
                            fill: false,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                    layout: {
                        padding: {
                            top: 20,
                            bottom: 0,
                            left: 0,
                            rigth: 0,
                        }
                    },
                }
            });
        },
    });
}
